;***************************************************************************
;	    I2C   AVR
;	     
;	AVR300: Software I2C Master Interface
;***************************************************************************
;   ,    I2C
   .equ IIC_PORT = PORTB		;,    I2C
   .equ IIC_DDR =  DDRB			;DDR,    I2C
   .equ IIC_PIN =  PINB			;,    I2C
   .equ SCL = PB5				;,    SCL
   .equ SDA = PB4				;,    SDA

;==========================================================================
; .       .

iic_init:
		CBI IIC_PORT,SDA		; 0    SDA
		CBI IIC_PORT,SCL		; 0    SCL
		CBI IIC_DDR,SDA			;    
		CBI IIC_DDR,SCL			;    
		
		ret

;==========================================================================
;			/   .

iic_restart:
	;	sbi	IIC_DDR,SCL			;    (0)
	;	cbi	IIC_DDR,SDA			;   (1)
	;	rcall iic_delay			; 
	;	cbi	IIC_DDR,SCL			;    (1)
	;	rcall iic_delay			; 

;==========================================================================
;				/   .
;     SDA   ,  
;   SCL       .
;
iic_start:
			;     
		cbi IIC_DDR,SDA			;
		cbi IIC_DDR,SCL			;
		rcall iic_delay			;  2,9 
			;   , 
		SBIS IIC_PIN,SDA		;    0 -
		rjmp iic_start			;  
		SBIS IIC_PIN,SCL		;    0 -
		rjmp iic_start			; ,  -
			;  ""
		SBI IIC_DDR,SDA			;    
		CBI IIC_PORT,SDA		; 0   
		rcall iic_delay			;  2,9 
		SBI IIC_DDR,SCL			;    
		CBI IIC_PORT,SCL		; 0   
		rcall iic_delay			;  2,9 
		
		ret
;==========================================================================
;				/   .
;    SDA    ,  
;   SCL       .

iic_stop:
			;  ""
		sbi IIC_DDR,SDA			;    0
			;  
		sbi IIC_DDR,SCL			;    0
		rcall iic_delay			;  2,9 
		cbi IIC_DDR,SCL			;    1
		rcall iic_delay			;  2,9 
			;  ACK    SDA
		cbi IIC_DDR,SDA			;   (1)
		rcall iic_delay			;  2,9 
		clc						;  
		sbic IIC_DDR,SDA		;    0 (.. ACK),
		ret						;
		sbic IIC_DDR,SCL		;     SCL 0, 
		ret						;
		sec						;   
		
		ret						;  /
		
;==========================================================================
;				/  1 .
;   ,    SDA ,    
;   SCL,   8 .   9- ,
;    ,    , 
;  ACK.       0  
;  SDA,     .
;   R16   . ;    
; C  SREG  1 - ACK  ,    0 -   ACK.

iic_send:
		push r16				;    
		push r17				;
		
		ldi r17,8				;  
			;  
s_bit:	rol r16 				;    
		brcs se1				;   = 0,  -
		sbi IIC_DDR,SDA			;      0
		rjmp del_wr				;   = 1,  -
se1:	cbi IIC_DDR,SDA			;      1
			
			;  
del_wr:	rcall iic_delay2		;
		cbi IIC_DDR,SCL			;    (1)
  		rcall iic_delay			;  2,9 
  		sbi IIC_DDR,SCL			;    (0)
  		rcall iic_delay			;  2,9 
  	
 		dec r17 				;  
  		brne s_bit				;   
  			;  ACK    SDA
  		cbi IIC_DDR,SDA			;  
 		rcall iic_delay			;  2,9 
  		cbi IIC_DDR,SCL			; 9-  (1)
  		rcall iic_delay			;  2,9 
		
		clc						;  
  		sbic IIC_PIN,SDA		;    1 (..  ACK) ,  -
  		sec						;   
		
 		sbi IIC_DDR,SCL			; 9-  (0)
 		rcall iic_delay			;  2,9 
		
		
 		pop r17					;  
  		pop r16					;
		
		ret
;==========================================================================
;					/  1 .
;   1  8,    SDA    
; .     SDA  (.. ACK)   
;,     9- .   
; ACK  ,       .
;    R16. ACK ,    
;  T  SREG.

iic_recieve:
		
		ldi r16,1				;   
		
loop_read:		; 
		; 
   		sbi IIC_DDR,SCL			; 0   
   		cbi IIC_DDR,SDA			;  
   		rcall iic_delay			;  2,9 
   		cbi IIC_DDR,SCL			; 1   
   		rcall iic_delay2		;
		;     
   		clc						;  
   		sbic IIC_PIN,SDA		;    1, - 
   		sec						;   
   		rol r16					;    r16
   		rcall iic_delay2		;
   		brcc loop_read			;    
			;  ACK
   		sbi IIC_DDR,SCL			; 0   
   		rcall iic_delay			;  2,9 
  		brts se0				;   ,  ACK,
   		cbi IIC_DDR,SDA			;  - 1   , .. NOACK
   		rjmp rd_				;   rd_
		
se0: 	sbi IIC_DDR,SDA			; 0    .. ACK
		
rd_:	clc						;  
  		rcall iic_delay			;  2,9 
  		cbi IIC_DDR,SCL			;   
  		rcall iic_delay			;  2,9 
  		
		sbi IIC_DDR,SCL			; 0   
		rcall iic_delay2		;
   		cbi IIC_DDR,SDA			;  
   		rcall iic_delay			;  2,9 
		ret						;
		
;========================================================================
; .  5.0  (   1.3us)
;  23  ( Fclk = 8 MHz  2,875 )
iic_delay:
			push r16
			ldi r16,6		;  
			dec r16			; 
			brne PC-1		;  
			pop r16
			ret
; ,       SDA 
;     SCL   
;    SCL (4.7 /2=2.4 ).
;========================================================================
iic_delay2:
			push r16
			ldi r16,2		;  
			dec r16			; 
			brne PC-1		;  
			pop r16
			ret
;==========================================================================

	.EXIT






;==========================================================================
;			/  2- . ()
;     R17:R16

iic_rd16:
		clt						;  
		clr r16					;
 rd_8:	mov r17,r16				; .   R17
		ldi r16,1				;   
		
loop_rd:		; 
		; 
   		sbi IIC_DDR,SCL			; 0   
   		cbi IIC_DDR,SDA			;  
   		rcall iic_delay			;  2,9 
   		cbi IIC_DDR,SCL			; 1   
   		rcall iic_delay2		;
		;     
   		clc						;  
   		sbic IIC_PIN,SDA		;    1, - 
   		sec						;   
   		rol r16					;    r16
   		rcall iic_delay2		;
   		brcc loop_rd			;    
   		brts s_ask				;   ,
   		set						;  
   		rjmp rd_8				;
   		
			;  ACK
s_ask:	sbi IIC_DDR,SCL			; 0   
   		rcall iic_delay			;  2,9 
  		brts se_0				;   ,  ACK,
   		cbi IIC_DDR,SDA			;  - 1   , .. NOACK
   		rjmp red				;   rd_
		
se_0: 	sbi IIC_DDR,SDA			; 0    .. ACK
		
red:	clc						;  
  		rcall iic_delay			;  2,9 
  		cbi IIC_DDR,SCL			;   
  		rcall iic_delay			;  2,9 
		
		sbi IIC_DDR,SCL			; 0   
		rcall iic_delay2		;
   		cbi IIC_DDR,SDA			;  
   		rcall iic_delay			;  2,9 
		
		ret						;
		
;========================================================================